/*
 * BlimaConfig.java
 *
 * Created on 11. Juli 2006, 12:30
 *
 */

package blima;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;


/**
 * Class for central configuration. 
 * 
 * Before closing the application call Config.getInstance().saveProperties() to
 * save all properties.
 * 
 * @author eeasw
 */
public class BlimaConfig
{
  
  /**
   * Creates a new instance of BlimaConfig
   */
  public BlimaConfig()
  {
    m_props = new Properties();
  }

  /** <code>m_props</code> is the container for all properties */
  private Properties m_props;
  
  /** <code>propertyFileName</code> is the name of the property file to use */
  public String propertyFileName = "blima.properties";

  /**
   * The instance of the DEFAULT Config.
   */
  private static BlimaConfig instance;
  /**
   * We create an instance when the Class is loaded and work with the instance.
   */
  static
  {
    BlimaConfig.instance = new BlimaConfig();
    instance.loadProperties();
  }

  /**
   * saves the configuration into a file
   */
  public void saveProperties()
  {
    File f = new File(propertyFileName);
    try
    {
      m_props.store(new FileOutputStream(f), "Config");
    } catch(FileNotFoundException e)
    {
      System.out.println("no file");
      e.printStackTrace();
    } catch(IOException e)
    {
      System.out.println("store didn't work");
      e.printStackTrace();
    }
  }
  
  /** loads the properties
   * 
   */
  private void loadProperties()
  {
    File f = new File(propertyFileName);
    if(!f.exists())
    {
      setDefaultProperties();
      try
      {
        f.createNewFile();
        m_props.store(new FileOutputStream(f), "Config");
        //System.out.println(f.getAbsolutePath()); // ist im ausfhrenden
        // verzeichnis
      } catch(IOException e2)
      {
        System.out.println("store didn't work");
        e2.printStackTrace();
      }

    }
    FileInputStream fis;
    try
    {
      fis = new FileInputStream(f);
      m_props.load(fis);
    } catch(FileNotFoundException e)
    {
      System.out.println("File not Found");
      e.printStackTrace();
    } catch(IOException e1)
    {
      System.out.println("load didn't work");
      e1.printStackTrace();
    }
  }

  /** sets default properties
   * 
   */
  private void setDefaultProperties()
  {
    // MARK set default config values here
    m_props.clear();
    m_props.setProperty("dataFolder", System.getProperty("user.dir") + File.separator + "data");
    m_props.setProperty("reportFolder", System.getProperty("user.dir") + File.separator + "report");
    m_props.setProperty("projectFolder", System.getProperty("user.dir") + File.separator + "project");
    m_props.setProperty("partycount", "0");
    //m_props.setProperty("", "");
    //m_props.setProperty("", "");
  }

  /**
   * Work only with this instance. It is static.
   * 
   * @return The static Config instance
   */
  public static BlimaConfig getInstance()
  {
    return BlimaConfig.instance;
  }

  //////////////////////////////////////////////////////////////////////////////////////
  /**
   * getter for all properties
   * @return proparty value
   * @param key identifying key "dataFolder" "reportFolder" "partycount"
   * @param defValue a default value
   */
  public static String getProperty(String key, String defValue)
  {
    Properties p = BlimaConfig.getInstance().m_props;
    String s = p.getProperty(key, defValue);
    return s;
  }
  public static String getProperty(String key)
  {
    Properties p = BlimaConfig.getInstance().m_props;
    String s = p.getProperty(key, "");
    return s;
  }

  //*/
  /**
   * sets a property
   * @param key identifying key
   * @param value the value
   */
  public static void setProperty(String key, String value)
  {
    BlimaConfig.getInstance().m_props.setProperty(key, value);
  }
}
