/*
 * BlimaProjekt.java
 *
 * Created on 17. Juli 2006, 11:46
 *
 * ein projekt. die dateien liegen im gleichnamigen unterordner
 */

package blima;

import aswtools.ASWTools;
import aswtools.FileTools;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Calendar;
import java.util.Date;
import java.util.Properties;
import java.util.Random;
import java.util.Vector;

/**
 * En Projekt besteht aus Name, Datum und Teilnehmerdaten
 * @author eeasw
 */
public class BlimaProjekt
{
  
  /**
   * Creates a new instance of BlimaProjekt
   * @param n der Name des Projekts
   */
  public BlimaProjekt(String n)
  {
    m_name = n;
    m_description = "";
    Calendar d = Calendar.getInstance();
    m_datum = d.get(Calendar.DATE)+"."+d.get(Calendar.MONTH)+"."+d.get(Calendar.YEAR);
    File fldr = projektfolder();
    if (!fldr.exists()) fldr.mkdir();
  }
  
  /**
   * konstruktor aus datei
   * @param f datei, in der die Projektdaten stehen
   */
  public BlimaProjekt(File f)
  {
    laden(f);
  }
  
  /**
   * Name "vom" Datum
   * @return textdarstellung
   */
  public String toString()
  {
    return m_name + " vom " + m_datum;
  }
  
  // Projektteilnehmer
  /**
   * Vector aus ProjektTeilnehmer
   */
  private Vector<ProjektTeilnehmer> m_leute = new Vector<ProjektTeilnehmer>();
  
  // wann das Projekt angelegt wurde
  /**
   * Datum als String
   */
  private String m_datum;
  
  //name des projekts
  /**
   * Name des Projekts, auch Name der Datei und des Datenordners
   */
  private String m_name;
  
  private String m_description;
  /**
   * teilnehmer hinzufgen
   * @param bt Blimateilnehmer
   */
  public void addTeilnehmer(BlimaTeilnehmer bt)
  {
    ProjektTeilnehmer pt = new ProjektTeilnehmer(bt);
    m_leute.addElement(pt);
  }
  
  /**
   * getter
   * @return Vector of ProjektTeilnehmer
   */
  public Vector getLeute()
  {
    return m_leute;
  }
  
  /**
   * den Teilnehmern fremde dateien zuordnen
   */
  public void mischen()
  {
    // TODO mische die Teilnehmer
    int k = m_leute.size();
    int t[] = ASWTools.mischen(k, true);
    // dateien kopieren und index sichern
    for (int i = 0; i < m_leute.size(); i++)
    {
      ProjektTeilnehmer pt = (ProjektTeilnehmer)m_leute.elementAt(i);
      ProjektTeilnehmer ptziel = (ProjektTeilnehmer)m_leute.elementAt(t[i]);
      pt.zielindex = t[i];
      pt.ziel = ptziel.name;
      ptziel.zielVon = pt.name;
      ptziel.zielVonindex = i;
      File fil = new File(ptziel.eigendatei);
      String datnam = fil.getName();
      int m = datnam.lastIndexOf(".");
      String destname = pt.name + datnam.substring(m);
      File dst = new File(projektfolder(), destname);
      FileTools.copyFileTo(fil, dst.getAbsolutePath());      
    }
  }
  
  /**
   * baut aus den Daten...
   * @return File fr die Projektdatei
   */
  private File projektdatei()
  {
    return new File(BlimaConfig.getProperty("projectFolder", "") + File.separator + m_name + ".blp");
  }
  
  /**
   * Baut aus den Daten...
   * @return File fr den Datenordner (wird nur beim Anlegen gebraucht)
   */
  private File projektfolder()
  {
    return new File(BlimaConfig.getProperty("projectFolder", "") + File.separator + m_name);
  }
  /**
   * Projektdatei schreiben
   */
  public void speichern()
  {
    Properties props = new Properties();
    props.setProperty("name", m_name);
    props.setProperty("beschreibung", m_description);
    props.setProperty("datum", m_datum);
    props.setProperty("anzahl", ASWTools.toStr(m_leute.size()));
    for (int i = 0; i < m_leute.size(); i++)
    {
      ProjektTeilnehmer p = (ProjektTeilnehmer)m_leute.elementAt(i);
      props.setProperty("t"+ASWTools.toStr(i), p.toSaveString());
    }
    File f = projektdatei();
    try
    {
      props.store(new FileOutputStream(f), "Projektdaten");
    }
    catch(FileNotFoundException e)
    {
      System.out.println("no file");
      e.printStackTrace();
    }
    catch(IOException e)
    {
      System.out.println("store didn't work");
      e.printStackTrace();
    }
  }
  
  /**
   * projektdatei lesen
   * @param f vollstndige dateibeschreibung
   */
  public void laden(File f)
  {
    //File f = projektdatei();
    FileInputStream fis;
    Properties props = new Properties();
    try
    {
      fis = new FileInputStream(f);
      props.load(fis);
    }
    catch(FileNotFoundException e)
    {
      System.out.println("File not Found");
      e.printStackTrace();
    }
    catch(IOException e1)
    {
      System.out.println("load didn't work");
      e1.printStackTrace();
    }
    m_name = props.getProperty("name");
    m_description = props.getProperty("beschreibung");
    m_datum = props.getProperty("datum");
    int k = ASWTools.toInt(props.getProperty("anzahl"));
    for (int i = 0; i < k; i++) m_leute.addElement(new ProjektTeilnehmer(props.getProperty("t"+ASWTools.toStr(i))));
  }
  
  /**
   * berprfen, ob ma einen Bericht generieren kann
   * @return true, wenn alle Teilnehmer effekte und aktion berichtet haben
   */
  public boolean checkDone()
  {
    boolean done = true;
    for (int i = 0; i < m_leute.size(); i++)
    {
      done &= ((ProjektTeilnehmer)m_leute.elementAt(i)).isDone();
    }
    return done;
  }
  
  public String createReport()
  {
    String s;
    StringBuffer sb1 = new StringBuffer(); // Kopf und bersicht
    StringBuffer sb2 = new StringBuffer(); // einzelreports und ende
    Calendar d = Calendar.getInstance();
    String datum = d.get(Calendar.DATE)+"."+d.get(Calendar.MONTH)+"."+d.get(Calendar.YEAR);
    sb1.append("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">");
    sb1.append("<html><head><title>");
    sb1.append(m_name);
    sb1.append("</title><link rel=\"stylesheet\" href=\"blima.css\" type=\"text/css\"></head><body>\n<h1>");
    sb1.append(toString());
    sb1.append("</h1><p>");
    sb1.append(m_description);
    sb1.append("</p><h2>");
    s = java.util.ResourceBundle.getBundle("blima/BlimaText").getString("repCloseDate");
    s = s.replace("%datum%", datum);
    sb1.append(s);
    sb1.append("</h2><ol>");

    // fr jeden eine berschrift
    for (int i = 0; i < m_leute.size(); i++)
    {
      ProjektTeilnehmer pt = (ProjektTeilnehmer)m_leute.elementAt(i);
      ProjektTeilnehmer tg = (ProjektTeilnehmer)m_leute.elementAt(pt.zielindex);
      ProjektTeilnehmer sr = (ProjektTeilnehmer)m_leute.elementAt(pt.zielVonindex);
      s = java.util.ResourceBundle.getBundle("blima/BlimaText").getString("repWorkedFor");
      s = s.replace("%participant%", pt.name);
      s = s.replace("%target%", pt.ziel);
      sb1.append("<li>");
      sb1.append(s);
      sb1.append("</li>");
      
      sb2.append("<hr><h3>");
      sb2.append(pt.name);
      sb2.append("</h3><ul><p><li>");
      s = java.util.ResourceBundle.getBundle("blima/BlimaText").getString("repAction");
      s = s.replace("%participant%", pt.name);
      s = s.replace("%target%", pt.ziel);
      sb2.append(s);
      sb2.append("</li><br><div class=\"aktion\">");
      sb2.append(pt.aktion);
      sb2.append("</div></p><p><li>");
      s = java.util.ResourceBundle.getBundle("blima/BlimaText").getString("repEffect");
      s = s.replace("%target%", pt.ziel);
      sb2.append(s);
      sb2.append("</li><br><div class=\"effekt\">");      
      sb2.append(tg.effekt);      
      sb2.append("</div></p><p><li>");
      s = java.util.ResourceBundle.getBundle("blima/BlimaText").getString("repAction");
      s = s.replace("%participant%", sr.name);
      s = s.replace("%target%", sr.ziel);
      sb2.append(s);
      sb2.append("</li><br><div class=\"aktion\">");
      sb2.append(sr.aktion);
      sb2.append("</div></p><p><li>");
      s = java.util.ResourceBundle.getBundle("blima/BlimaText").getString("repEffect");
      s = s.replace("%target%", pt.name);
      sb2.append(s);
      sb2.append("</li><br><div class=\"effekt\">");      
      sb2.append(pt.effekt);      
      sb2.append("</div></p></ul>");      
    }        
    sb2.append("</body></html>");
    sb1.append("</ol>");
    sb1.append(sb2);
    return sb1.toString();
  }
  
  public void cleanup()
  {
    // ordner mit dateien lschen
    FileTools.deleteAll(projektfolder());
    // projektdatei lschen
    projektdatei().delete();
  }

  String getName()
  {
    return m_name;
  }

  public String getDescription()
  {
    return m_description;
  }

  public void setDescription(String description)
  {
    m_description = description;
  }
}
